#!/bin/ash
# This script reconfigures the RLM with the info
# from the $tuples.

set -x

####################################
#    FILES CHECKING
####################################

ifup="/sbin/ifup"
ifdown="/sbin/ifdown"
IFCONFIG="/sbin/ifconfig eth0"

cmd_hostname="/bin/hostname"
do_firewall=0
fw_printenv="/usr/local/bin/fw_printenv"
tuples="/mnt/logs/etc/rlm_config_from_filer"
interfaces="/mnt/logs/etc/interfaces"
tmp_if="/tmp/interfaces"
route_info="/tmp/route_info"

hostname="/mnt/logs/etc/hostname"
tmp_dns="/tmp/config/etc/resolv.conf"
dns="/mnt/logs/etc/resolv.conf"

firewall="/etc/init.d/firewall"
IPv4="/sbin/ip"
log="/var/log/messages"
date=`date | sed "s/[A-Z]\{3\} [0-9]\{4\}//"`
tmp_ps="/tmp/rlm_ps"
IPTABLES="/sbin/iptables"
IP6TABLES="/sbin/ip6tables"
sp_ssh_conf="/mnt/logs/etc/sp_ssh_conf"

rm -f $tmp_if

####################################
#    UP-CONVERT DNS CONFIG FILE     
# This section (and only this) should be run if    
# called during init i.e., with 
# "dns_config" as command line parameter
# Normal invocations without any additional 
# parameters will skip this section.                
####################################
if [ $# -eq 1 -a $1 = "dns_config" ]; then
   if [ -e $dns -a -e $tuples ]; then 
      tmp_dns1="/tmp/resolv.conf"
      grep -q "sp-auto-generated" $dns
      reconfig_needed=$?
      if [ $reconfig_needed -eq 1 ]; then
         dns_domainname=`sed -n "s/^dns.domainname //p" $tuples`
   
         if [ "$dns_domainname" != "" ]; then
            echo "# sp-auto-generated by rlm_conf.sh at init time" > $tmp_dns1
            echo "search $dns_domainname" >> $tmp_dns1
            cat $dns >> $tmp_dns1
            mv -f $tmp_dns1 $dns
         fi
   
      fi
   fi
exit 0
fi


#############################################
#  Generate process id (configd)
#############################################

ps > $tmp_ps
pid=`awk '/configd/ {print $1}' $tmp_ps`
loginfo="$date (none) configd[$pid]:"
rm -f $tmp_ps
err=0
dhcp=0
is_v6_global_present=0

if [ -e $tuples ]; then
	chmod 0666 $tuples
fi



#############################################
#    CHECK IF FIREWALL NEEDS TO BE RESTARTED
#############################################

is_dhcp_enabled=`awk '/dhcp/ {if ($2 == 1) print "1"; else print "0"}' $tuples`
is_ipv4_enabled=`awk '/ipv4_disable/ {if ($2 == 1) print "0"; else print "1"}' $tuples`
is_ipv6_enabled=`awk '/ipv6_disable/ {if ($2 == 1) print "0"; else print "1"}' $tuples`
is_ipv6_global_present=`awk '/ip6_addr/ {print $2}' $tuples`
is_ipv6_ra_disabled=`awk '/ra_disable/ {print $2}' $tuples`
if [ -z $is_ipv4_enabled ]
then	
	is_ipv4_enabled=1
fi

if [ -z $is_ipv6_enabled ]
then	
	is_ipv6_enabled=0
fi

if [ -z $is_dhcp_enabled ]
then	
	is_dhcp_enabled=0
fi

if [ -z $is_ipv6_global_present ]
then	
	is_ipv6_global_present=0
else
	is_ipv6_global_present=1
fi

if [ -z $is_ipv6_ra_disabled ]
then
	is_ipv6_ra_disabled=0
fi

if [ -e $tuples ]
then
	# read the tuples files and create the interface ($tmp_if) file
	`awk -f /usr/local/bin/rlm_conf_awk_rules \
	     dhcp_enabled=$is_dhcp_enabled \
	     ipv4_enabled=$is_ipv4_enabled \
	     ipv6_enabled=$is_ipv6_enabled \
	     ipv6_global_present=$is_ipv6_global_present \
	     -v tmp_if_file=$tmp_if \
	     $tuples`
	
	is_v6_global_present=`grep -c "inet6 static" $tmp_if`	
	
	if [ $is_ipv6_enabled -eq 0 ]
	then	
		echo "#ipv6 disabled" >> $tmp_if
	else
		if [ $is_v6_global_present -eq 0 ]
		then
			echo "#ipv6 enabled without Global address" >> $tmp_if
		fi
		if [ $is_ipv6_ra_disabled -eq 1 ]
		then
			echo "#ipv6 enabled without Router Assigned address" >> $tmp_if
		fi
	fi

	#compute the diff
	diff $interfaces $tmp_if

	if [ $? = 1 ]; then {
		do_firewall=1;
	}
	fi
fi


##############################################################
#    CHECK IF BOTH INTERFACES AND TUPLES FILES ARE NOT PRESENT
##############################################################

if [ ! -e $interfaces -a ! -e $tuples ]
then
	is_dhcp_enabled=0
	echo "# This file is generated by RLM upon request from Ontap." > tmp_if
	echo "# Do NOT modify this file manually." >> tmp_if
	echo "auto lo" >> tmp_if
	echo "iface lo inet loopback" >> tmp_if
	echo "auto eth0" >> tmp_if
	echo "iface eth0 inet static" >> tmp_if
	echo "   address 192.168.220.2" >> tmp_if
	echo "   netmask 255.255.255.0" >> tmp_if
	echo "   gateway 192.168.220.1" >> tmp_if
	echo "#ipv6 disabled" >> $tmp_if
	do_firewall=1
fi

####################################################
#    CHECK IF GLOBAL V6 ADDRESS NEEDS TO BE DELETED
####################################################

if [ $is_ipv6_enabled -eq 1 ]
then
	new_ip6_addr=`awk '/ip6_addr/ {print $2}' $tuples`
	inet_val=`awk '/eth0 inet / {print $4}' $interfaces`

	if [ -z $inet_val -o $inet_val == "dhcp" ]
	then
		# interfaces file is going to have just 1 line for 
		# each of address and netmask fields
		cur_ip6_addr=`awk '/address / {print $2}' $interfaces`
		cur_net6=`awk '/netmask / {print $2}' $interfaces`
	else
		# interfaces file is going to have 2 lines for 
		# each of address and netmask fields (v4 and v6)
		# pick the second line in this case
		cur_ip6_addr=`awk '/address / {if (i==1) print $2; i++;}' $interfaces`
		cur_net6=`awk '/netmask / {if (i==1) print $2; i++;}' $interfaces`
	fi

	if [ -z $new_ip6_addr ]
	then
		if [ -n "$cur_ip6_addr" ]
		then
			`$IFCONFIG del "$cur_ip6_addr"/"$cur_net6"`			
		fi
	fi
fi

###########################################################################
#    sp_ssh_config tuple is not present in ONTAP version < IC.6 and also in
#       versions that doesn't support RLM/SP SSH access control feature
#    in that case flush ssh_pkts chain in the firewall
############################################################################
is_ssh_access_enabled=`grep -c sp_ssh_config $tuples`
if [ $is_ssh_access_enabled -eq 0 ]
then
	rm -f $sp_ssh_conf
	$IPTABLES -F ssh_pkts

	#Following 2 rules create a 'recent' list of recently seen IP addresses.
	#If new SSH connection requests are made from the same IP in a short
	#interval, it is likely a DOS attack. The following rule allows 10 SSH
	#requests from the same IP within a minute and then drops new requests.

	$IPTABLES -A ssh_pkts -p TCP -i eth0 --dport 22 -m state \
	    --state NEW -m recent --set
	$IPTABLES -A ssh_pkts -p TCP -i eth0 --dport 22 -m state \
	    --state NEW -m recent --update --seconds 60 --hitcount 10 -j DROP

	#Following rule rate-limits new incoming ssh connection requests to
	#an average of 20 per minute, with an initial burst of 20 connections
	$IPTABLES -A ssh_pkts -p TCP -i eth0 -s 0/0 --dport 22 --syn \
	    -m state --state NEW -m limit --limit 20/minute --limit-burst 20 -j ACCEPT

	$IP6TABLES -F ssh_pkts
	$IP6TABLES -A ssh_pkts -p tcp -i eth0 -s 0/0 --dport 22 -j ACCEPT
fi
####################################
#    HOSTNAME CONFIG
####################################

$fw_printenv | sed -n "s/serial#=//p" > $hostname || {
    echo "$loginfo failed to save hostname" 1>>$log
    $err=42	
    exit $err;
}

$cmd_hostname -F $hostname || {
    echo "$loginfo failed to config hostname" 1>>$log
    $err=42	
    exit $err;
}

####################################
#    DNS RESOLUTION
####################################
# dns is optional so do not check the copy
if [ -e $tmp_dns -a -e $tuples ]; then
   dns_domainname=`sed -n "s/^dns.domainname //p" $tuples`
   if [ "$dns_domainname" != "" ]; then
      echo "# sp-auto-generated by rlm_conf.sh invoked by configd" > $dns
      echo "search $dns_domainname" >> $dns
      cat $tmp_dns >> $dns
   else 
      cp $tmp_dns $dns
   fi
fi


####################################
#    IP TABLES
####################################
# Rewrite the file anyway to get the correct format
if [ -e $tmp_if ]
then
	cp $tmp_if $interfaces || {
	   echo "$loginfo can't save $interfaces" 1>>$log
		err=34	
	   exit $err
	}
fi

/usr/local/bin/net_link_up.sh;
if ! [ $? = 1 ]; then {
   echo "# no link"  >> $interfaces
   err=28	
   exit $err
}
fi

is_v4_static_present=`cat /mnt/logs/etc/interfaces | grep -c "inet static"`
is_eth0_present=`cat /mnt/logs/etc/interfaces | grep -c eth0`
if [ $do_firewall -eq 1 ]; then	
		
		# Before running the firewall, the eth0 interface needs to be
		# brought down and then up. 
		#
		# There are 4 statements below that do this job
		# 1. ifdown eth0
		# 2. ifconfig eth0 down
		# 3. ifup eth0
		# 4. ifconfig eth0 up
		#
		# 1 & 3 are always run
		# 2 & 4 are optional and are run during just this case,
		# Whenever there 
		#	- IPv6 is enabled
		#	- No v6 entry in the /mnt/logs/etc/interfaces file
		#	- DHCPv4 or no IPv4 entry is present in /mnt/logs/etc/interfaces 
		#
		# Reasons:
		#
		# 1. ifdown eth0 and ifup eth0  
		#	--------------------------		
		#   Will only work on addresses present in the /etc/network/interfaces
		#   file. And since LLA and RA is brought up by the kernel
		#   and this info is not present in the interfaces file, we
		#   need the below 2 statements to bring up RA and LLA of RLM
		#
		# 2. ifconfig eth0 down and ifconfig eth0 up
		#	----------------------------------------   
		# 	These 2 have no effect on IPv4 address but will bring back IPv6 
		# 	addresses which are native to the kernel and not present in the
		#   /etc/network/interfaces file
		#
		# 3. Same Ordering of these statments are important as well.

		# NOTE: Kernel brings up v6 interfaces only in 2 cases:
		# 1. When interfaces file have Global v6 Address, through "ifup eth0"
		# 2. When interfaces file does not have any v6 entry, but v6 is enabled
		# on RLM, then "ifconfig eth0 up" will bring up the native IPv6 
		# addresses back to life.
		# Burt reference 361431
	
	$ifdown eth0
	if [ $? -eq 1 -a $is_eth0_present -eq 1 ]; then 
		echo "$loginfo ifdown failed to bring down eth0" 1>>$log
		err=36
		exit $err
	fi
	
	if [ $is_ipv6_enabled -eq 1 -a $is_v6_global_present -eq 0 -a $is_v4_static_present -eq 0 ]; then
		$IFCONFIG down
		if [ $? -eq 1 ]; then 
    		echo "$loginfo ifconfig failed to bring down eth0" 1>>$log
			err=38
			exit $err	
		fi
	fi
	
	# If there is a UDHCPC client running before, kill it completely
	# Refer burt 344656 for more information
	if [ -e "/var/run/udhcpc.eth0.pid" ]
	then
		kill `cat /var/run/udhcpc.eth0.pid` 2>/dev/null
	fi

	$ifup eth0
	if [ $? -eq 1 -a $is_eth0_present -eq 1 ]; then { 
		echo "$loginfo ifup failed to bring up eth0" 1>>$log	
	}
	fi
	
	if [ $is_ipv6_enabled -eq 1 -a $is_v6_global_present -eq 0 -a $is_v4_static_present -eq 0 ]; then
		$IFCONFIG up
		if [ $? -eq 1 ]; then 
    		echo "$loginfo ifconfig failed to bring up eth0" 1>>$log
			err=39
			exit $err
		fi
		# ifconfig eth0 down and ifconfig eth0 up commands remove the
		# default entry for route.  Following code adds them back.
		if [ $is_ipv4_enabled -eq 1 ]
		then
			if [ $is_dhcp_enabled -eq 1 ]
			then
				gateway=`awk '/gateway/ {print $2}' $route_info`
				interface=`awk '/interface/ {print $2}' $route_info`
			else	
				gateway=`awk '/gateway/ {print $2}' $tuples`
				interface=`awk '/inet static/ {print $2}' $interfaces`
			fi
			route add default gw $gateway $interface
		fi			
	fi

	# Set this value if the RA needs to be disabled.
	# Even though this will not delete the RA IPv6 address present in ifconfig,
	# RLM will no longer accpet any new RA addresses after this
	# Deletion of the RA will be done in the firewall

	if [ $is_ipv6_enabled -eq 1 ]
	then
		if [ -e "/proc/sys/net/ipv6/conf/eth0/accept_ra" ]
		then
			if [ $is_ipv6_ra_disabled -eq 1 ]
			then
				echo "0" > "/proc/sys/net/ipv6/conf/eth0/accept_ra"
			else 
				echo "1" > "/proc/sys/net/ipv6/conf/eth0/accept_ra"
			fi
		else
			echo "rlm_conf.sh - /proc/sys/net/ipv6/conf/eth0/accept_ra, file not found" >> $log
		fi
	fi
		
	$firewall restart || {
		echo "$loginfo firewall starter failed, err: $ret" 1>>$log
		err=43
		exit $err
	}
fi

echo "$loginfo $0 completed, err = $err" 1>>$log
exit $err
