#!/bin/ash
#
# This script configures speed and duplex for "rlm config" command.
# Copyright (c) 2012 NetApp, Inc
# All rights reserved.

log="/var/log/messages"
rlm_port_config="/mnt/logs/etc/rlm_port_config"
date=`date | sed "s/[A-Z]\{3\} [0-9]\{4\}//"`
loginfo="$date rlmport_config.sh :"

if ! [ -e $rlm_port_config ]; then
	echo "$loginfo file $rlm_port_config not present " 1>>$log
	exit 1;
fi

media_setting=`awk '/^mediatype/ {print $2}' $rlm_port_config`

if [ -z $media_setting ]; then
	echo "$loginfo mediatype tuple value not present " 1>>$log
	exit 1;
fi

if [ "$media_setting" = "100tx-fd" ]; then
	echo "$loginfo Forcing the RLM Ethernet configuration to 100 Mbps and full duplex" >> $log
	/usr/local/bin/ethtool -s eth0 autoneg off speed 100 duplex full
elif [ "$media_setting" = "100tx-hd" ]; then
	echo "$loginfo Forcing the RLM Ethernet configuration to 100 Mbps and half duplex" >> $log
	/usr/local/bin/ethtool -s eth0 autoneg off speed 100 duplex half
elif [ "$media_setting" = "10t-fd" ]; then
	echo "$loginfo Forcing the RLM Ethernet configuration to 10 Mbps and full duplex" >> $log
	/usr/local/bin/ethtool -s eth0 autoneg off speed 10 duplex full
elif [ "$media_setting" = "10t-hd" ]; then
	echo "$loginfo Forcing the RLM Ethernet configuration to 10 Mbps and half duplex" >> $log
	/usr/local/bin/ethtool -s eth0 autoneg off speed 10 duplex half
elif [ "$media_setting" = "auto" ]; then
	echo "$loginfo RLM Ethernet auto-negotiation is enabled" >> $log
	/usr/local/bin/ethtool -s eth0 autoneg on
else
	echo "$loginfo Wrong argument value: RLM Ethernet auto-negotiation is enabled" >> $log
	/usr/local/bin/ethtool -s eth0 autoneg on
fi

/bin/sleep 2
echo miitool eth0 >> $log
echo ============= >> $log
/usr/local/bin/miitool eth0 >> $log

exit 0;
